class B { 
    public virtual  void O() { System.Console.WriteLine("B.O()"); }
    public virtual  void V() { System.Console.WriteLine("B.V()"); }
}
class D : B {
    public override void O() { System.Console.WriteLine("D.O()"); }
}
class TestVirtualOverride {
    public static void Bind(B b) { 
        b.O(); 
        b.V(); 
    }
    public static void Main() {
        Bind( new B() );
        Bind( new D() );

        new D().O();
        new D().V();
    }
}

/*
rb           vtbls (virtual or method tables)

      
rb
    B |   vptr -----> 0 | o------> B.O()
      |   B fields    1 | o------> B.V()


rd
D | B |   vptr -----> 0 | o------> D.O()
  |   |   B fields    1 | o------> B.V()
  |       D fields            


rb = reference to a B object            
rd = reference to a D object
vptr = pointeur to a virtual table

Giving a clue of how it works under the hood help to gain
a better understanding of how virtual methods are implemented.
Virtual methods are implemented through a table of pointers
to functions (methods) called (virtual) method table, the mtbl.

Every object that contains at least one virtual method contains
a pointer to a mtbl. Callling a virtual method is indexing into
that table and calling a method through a pointer to the function.

When an object is instantiated from a class that contains at least
one virtual method, an internal method table is allocated and
pointed by a pointer mtbl as shown below:


an internal structure contains
its data fields. If its class .
*/
