    using System;

    public class DeviceException : Exception {
        public DeviceException() { }
        public DeviceException(string msg) : base(msg) { }
        public DeviceException(string msg, Exception inner) : base(msg, inner) {}
    }

    public class Device {
        // where an exception is thrown.
        public byte Read() {
            byte b = 0;

            if (!status)
                throw new DeviceException("Cannot read.");
            //...
            return b;
        }

        // where an exception is thrown (by the runtime system).
        public void Process()   {
            int num = 2;
            int den = 0;
            // ...

            // The next statement will generate 
            // an arithmetic exception: DivideByZeroException.

            System.Console.WriteLine( num/den );
            System.Console.WriteLine( "done." );
        }
        private bool status = false;
    }

    public class LowLevelDeviceDriver {
        public LowLevelDeviceDriver() { dev = new Device(); }

        // When the exception is not handled and propagated up to the caller.
        public void Init() {
            dev.Read();     // May throw a DeviceException.
            dev.Process();  // May throw a DivideByZeroException.
        }
        private Device dev;
    }

    public class DeviceManager1 {
        public DeviceManager1() { dd = new LowLevelDeviceDriver(); }

        public void Init() {  // When the exception is handled.
            try {
                dd.Init();
            } catch(DeviceException e) {
                Console.WriteLine("Handled in DeviceManager1 [{0}]", e);
            } catch(DivideByZeroException e) {
                Console.WriteLine("Handled in DeviceManager1 [{0}]", e);
            }
        }
        private LowLevelDeviceDriver dd;
    }

    public class DeviceSecureManager {
        public DeviceSecureManager() { dm = new DeviceManager1(); }

        // When the exception is handled by a subobject DeviceManager.
        public void Init() {
            dm.Init();
        }
        private DeviceManager1 dm;
    }

    public class HighLevelDeviceDriver {
        public HighLevelDeviceDriver() { lldd = new LowLevelDeviceDriver(); }

        // When the exception is partially handled by HighLevelDeviceDriver and
        // is rethrown (or relayed) to DeviceManager2.
        public void Init() {
            try {
                lldd.Init();
            } catch(DeviceException e) {
                Console.WriteLine("Handled in HighLevelDD and rethrown[{0}]", e);
                // Some additional handling here...
                throw new DeviceException(e.Message, e);
            }
        }
        private LowLevelDeviceDriver lldd;
    }

    public class DeviceManager2 {
        public DeviceManager2() { dd = new HighLevelDeviceDriver(); }

        // When the exception is handled by several catchers that rethrow.
        public void Init() {
            try {
                dd.Init();
            } catch(DeviceException e) {
                Console.WriteLine("Handled in DeviceManager2 [{0}]", e);
            }
        }
        private HighLevelDeviceDriver dd;
    }
    
