using System;
using System.Text;

namespace BitIndexer {
    public class IntBits {
        public bool this [int index] {
            get {
                if (index < 0 || index >= BitsPerInt) 
                    throw new ArgumentOutOfRangeException();

                return (bits & (1 << index)) != 0; 
            }
            set { 
                if (index < 0 || index >= BitsPerInt) 
                    throw new ArgumentOutOfRangeException();

                if (value) 
                    bits |= (1 << index);
                else
                    bits &= ~(1 << index);
            }
        }
        public override string ToString() {
            StringBuilder s = new StringBuilder("[ ");
            for (int n = 0; n < BitsPerInt; n++)
                 s.Append(this[n] ? "1 ": "0 ");
            s.Append("]");
            return s.ToString();
        }
        private       int bits;
        private const int BitsPerInt = 32;
    }
    
    public class TestBitIndexer {
        public static void Main() {
            IntBits  ibits = new IntBits();

            ibits[6] = true;             // set
            bool peek = ibits[2];        // get
            ibits[6] = ibits[2] || peek; // get and set

            Console.WriteLine( ibits.ToString() );
        }
    }
}
