using System;
using System.Reflection;
using Attributes;

namespace MyNamespace {
    class TypesWithInvariants {
        public static void Main() {
            Assembly assembly = Assembly.Load("UsingAttributes");
            Module   module   = assembly.GetModule("UsingAttributes.dll");

            Type[] types = module.FindTypes(Module.FilterTypeName, "*");
            
            Console.WriteLine("Types found in 'UsingAttributes.dll': ");
            foreach(Type t in types)
                Console.Write("{0} ", t.Name);
            Console.WriteLine();

            Console.WriteLine("\nFor every type, list all public methods:");
            foreach(Type t in types)
                foreach (MethodInfo m in t.GetMethods()) 
                    Console.WriteLine("{0} has {1}", t, m.Name);
                
            Console.WriteLine("\nFor every type, list all non-public instance methods:");
            foreach(Type t in types)
                foreach (MethodInfo m in t.GetMethods(BindingFlags.Instance | 
                                                      BindingFlags.NonPublic)) 
                    Console.WriteLine("{0} has {1}", t, m.Name);

            Console.WriteLine("\nTypes that have invariant attributes:");
            foreach (Type t in types) 
                foreach (MethodInfo m in t.GetMethods(BindingFlags.Instance | 
                                                      BindingFlags.NonPublic)) 
                    foreach (Attribute attrib in m.GetCustomAttributes(true)) {
                        if (attrib is InvariantAttribute)
                            Console.WriteLine("{0} has {1}", t, m.Name);
                    }
         }
    }
}
